# Functions that build a data package using GO data
#

GOPkgBuilder <- function(pkgName = "GO", pkgPath, version = "1.2.1",
                         goUrl = getSrcUrl("GO", xml = TRUE),
                         llUrl = getSrcUrl("ll"),
                         author = list(author = "who",
                         maintainer = "who@email.com"),
                         repList = getRepList("go")){
    print("It may take me a while to process the data. Be patient!")
    if(missing(pkgPath)){
        print(paste("No path specified. The resulting package will be",
                    "stored in", getwd()))
        pkgPath <- getwd()
    }

    makeSrcInfo()
    createEmptyDPkg(pkgName, pkgPath, force = TRUE)

    # A matrix to keep GO ids and group (CC, BP, MF) information
#    goNGroup <- cbind(getTopGOid("GO"), "GO")
    go <- GO(goUrl)
    goData <- appendObsolete(readData(go, xml = TRUE))
    # Add a category column
    goData <- cbind(goData, rep(NA, nrow(goData)))
    goData[goData[,1] == "GO:0003673", ncol(goData)] <- "GO root"
    # Process data for MF, CC, and BP and write the data to parents
    # and ontology term data file
    goGroups <- getGOGroupIDs()
    for(i in names(goGroups)){
        goids <- c(as.vector(goGroups[i]),
                   getOffspringNodes(goGroups[i], goData))
        # Map GO ids to the three ontology categories. This file will
        # be used later to create the go category environment
        # goNGroup <- rbind(goNGroup, cbind(goids, i))
        goData[is.element(goData[,1], goids), ncol(goData)] <- i
        temp <- goData[is.element(goData[,1], goids),]
        # Write GO id to parent GO id data file
        saveMat(temp[, c(1, 4)], pkgName = pkgName, pkgPath = pkgPath,
                    envName = paste(i, "PARENTS", sep = ""),
                    fun = twoStepSplit)
        # Write Ontology file
        # temp <- goData[is.element(goData[,1], c(getTopGOid("GO"), goids)),]
        #saveMat(temp[, c(1, 3)], pkgName = pkgName, pkgPath = pkgPath,
        #            envName = paste(i, "ID2TERM", sep = ""),
        #            fun = splitEntry)
        # Write GO id to children GO id data file
        temp <- sapply(goids, getChildNodes, goData = goData)
        temp <- sapply(temp, paste, sep = "", collapse = ";")
        # Write GO id to parent GO id data file
        saveMat(cbind(names(temp), temp), pkgName = pkgName,
                pkgPath = pkgPath, envName = paste(i, "CHILDREN", sep = ""),
                fun = splitEntry)
        #temp <- sapply(goids, getAncestors, goData = goData, keepTree = TRUE)
        #saveList(temp, pkgName = pkgName, pkgPath = pkgPath,
        #         envName = paste(i, "ANCESTOR", sep = ""))
        #temp <- sapply(goids, getOffspringNodes, goData = goData,
        #               keepTree = TRUE)
        #saveList(temp, pkgName = pkgName, pkgPath = pkgPath,
        #         envName = paste(i, "OFFSPRING", sep = ""))
    }
    # Write the mapping between GO ids and category information
    ll <- LL(srcUrl = llUrl, parser = file.path(.path.package("AnnBuilder"),
                                   "scripts", "llNGOParser"))
    llMapping <- try(parseData(ll, ncol = 2, fromWeb = TRUE))
    # Do LL2GO ids
    saveMat(llMapping, pkgName = pkgName, fun = twoStepSplit,
                 pkgPath = pkgPath, envName = "LL2GO")
    # Do GO2LL
    go2LL <- reverseMap4GO(llMapping, sep = ";", "ll2GO")
    saveMat(go2LL, pkgName = pkgName, fun = twoStepSplit,
                 pkgPath = pkgPath, envName = "GO2LL")
    #go2All <- mapGO2AllProbe(go2LL, goData, "", sep = ";", all = TRUE)
    #saveMat(cbind(names(go2All), go2All), fun = twoStepSplit,
    #        pkgName = pkgName, pkgPath = pkgPath, envName = "ALLLOCUSIDS")
    #go2All <- go2All[!is.na(go2All)]
    #ll2AllGO <- reverseMap4GO(cbind(names(go2All), go2All), ";", "GO2LL")
    #saveMat(ll2AllGO, fun = twoStepSplit,
    #        pkgName = pkgName, pkgPath = pkgPath, envName = "LL2ALLGO")
    #Write the mapping between GO ids and category information
    #saveMat(goData[, c(1, ncol(goData))], pkgName = pkgName,
    #            pkgPath = pkgPath, envName = "CATEGORY")
    #Write the mappings between GO ids and GO terms
    saveMat(cbind(goData[,1], paste(goData[,3], goData[,4], sep = "@")),
            fun = twoStepSplit, pkgName = pkgName, pkgPath = pkgPath,
            envName = "TERM")

    writeDocs(pkgName, pkgPath, version, author, repList)
}

writeDocs <- function(pkgName, pkgPath, version, author, repList){
    # Write man pages and other files
    writeDescription(pkgName, pkgPath, version, author)
    copyTemplates(repList, "GO", pkgName, pkgPath)
    copySubstitute(file.path(.path.package("AnnBuilder"), "templates",
                             "GENERAL.Rd"), file.path(pkgPath, pkgName,
                             "man", paste(pkgName, ".Rd", sep = "")),
                              list(PKGNAME = pkgName, SOURCENBUILT =
                                   paste("\n\tGene Ontology",
                                         "\\\\url{", repList[["GOSOURCE"]],
                                         "}. Built", repList[["GOBUILT"]],
                                         "\n\n\tLocusLink \\\\url{",
                                         repList[["LLSOURCE"]], "}. Built",
                                         repList[["LLBUILT"]])), "#")
    writeZZZ(pkgPath, pkgName)
    writeFun(pkgPath, pkgName)
    getDPStats("", pkgName, pkgPath)
    writeMan4QC(pkgName, pkgPath)
}

copyTemplates <- function(repList, pattern, pkgName, pkgPath,
                                              replaceBy = NULL){
    templates <- gsub(pkgName, pattern, gsub("\\.rda$", ".Rd",
                     list.files(file.path(pkgPath, pkgName, "data"))))
    #templates <- gsub(pkgName, pattern, manPages)
    #templates <- list.files(file.path(.path.package("AnnBuilder"),
    #                 "templates"), pattern, full.names = TRUE)

    for(i in templates){
        if(is.null(replaceBy)){
            rdName <- file.path(pkgPath, pkgName, "man",
                            gsub(pattern, pkgName, i))
        }else{
            rdName <- file.path(pkgPath, pkgName, "man",
                            gsub(pattern, replaceBy, i))
        }
        copySubstitute(file.path(.path.package("AnnBuilder"),
                                 "templates", i), rdName, repList, "#")
    }
}

getRepList <- function(what){
    switch(toupper(what),
           GO = return(list(LLSOURCE = getSrcUrl("ll"),
                            GOSOURCE = getSrcUrl("go"),
                            LLBUILT = getSrcBuilt("ll"),
                            GOBUILT = getSrcBuilt("go"),
                            DATE = date())),
           stop(paste("Unknown argument", what)))
}

appendObsolete <- function(goData){

    obsoletes <- getOffspringNodes(goData[grep("obsolete",
                                               goData[, 3]), 1], goData)
    ontology <- goData[,3]
    goData[is.element(goData[,1], obsoletes), 3] <-
        paste(goData[is.element(goData[,1], obsoletes), 3],
              "(Obsolete)")

    return(goData)
}












